@@ -10,6 +10,7 @@ from datetime import datetime
 from operator import itemgetter
 from xml.etree import ElementTree
 from uuid import uuid4
+from copy import deepcopy
 
 from label_studio_converter import Converter
 
@@ -18,6 +19,7 @@ from label_studio.utils.analytics import Analytics
 from label_studio.utils.models import ProjectObj, MLBackend
 from label_studio.utils.exceptions import ValidationError
 from label_studio.utils.io import find_file, delete_dir_content, json_load
+from label_studio.utils.validation import is_url
 from label_studio.tasks import Tasks
 
 logger = logging.getLogger(__name__)
@@ -39,8 +41,9 @@ class Project(object):
         self.context = context or {}
 
         self.tasks = None
-        self.label_config_line, self.label_config_full, self.input_data_tags = None, None, None
+        self.label_config_line, self.label_config_full, self.parsed_label_config, self.input_data_tags = None, None, None, None  # noqa
         self.derived_input_schema, self.derived_output_schema = None, None
+
         self.load_tasks()
         self.load_label_config()
         self.load_derived_schemas()
@@ -48,11 +51,13 @@ class Project(object):
         self.analytics = None
         self.load_analytics()
 
-        self.project_obj, self.ml_backend = None, None
+        self.project_obj = None
+        self.ml_backends = []
         self.load_project_ml_backend()
 
         self.converter = None
         self.load_converter()
+        self.max_tasks_file_size = 250
 
     def load_tasks(self):
         self.tasks = {}
@@ -73,6 +78,7 @@ class Project(object):
     def load_label_config(self):
         self.label_config_full = config_comments_free(open(self.config['label_config']).read())
         self.label_config_line = config_line_stripped(self.label_config_full)
+        self.parsed_label_config = parse_config(self.label_config_line)
         self.input_data_tags = self.get_input_data_tags(self.label_config_line)
 
     def load_derived_schemas(self):
@@ -97,17 +103,39 @@ class Project(object):
         collect_analytics = bool(collect_analytics)
         self.analytics = Analytics(self.label_config_line, collect_analytics, self.name, self.context)
 
+    def add_ml_backend(self, params, raise_on_error=True):
+        ml_backend = MLBackend.from_params(params)
+        if not ml_backend.connected and raise_on_error:
+            raise ValueError('ML backend with URL: "' + str(params['url']) + '" is not connected.')
+        self.ml_backends.append(ml_backend)
+
+    def remove_ml_backend(self, name):
+        # remove from memory
+        remove_idx = next((i for i, b in enumerate(self.ml_backends) if b.model_name == name), None)
+        if remove_idx is None:
+            raise KeyError('Can\'t remove ML backend with name "' + name + '": not found.')
+        self.ml_backends.pop(remove_idx)
+
+        # remove from config
+        config_params = self.config.get('ml_backends', [])
+        remove_idx = next((i for i, b in enumerate(config_params) if b['name'] == name), None)
+        if remove_idx is not None:
+            config_params.pop(remove_idx)
+        self.config['ml_backends'] = config_params
+        self._save_config()
+
     def load_project_ml_backend(self):
         # configure project
         self.project_obj = ProjectObj(label_config=self.label_config_line, label_config_full=self.label_config_full)
-        # configure machine learning backend
-        ml_backend_params = self.config.get('ml_backend')
-        if ml_backend_params:
-            self.ml_backend = MLBackend.from_params(ml_backend_params)
-            self.project_obj.connect(self.ml_backend)
+
+        # configure multiple machine learning backends
+        self.ml_backends = []
+        ml_backends_params = self.config.get('ml_backends', [])
+        for ml_backend_params in ml_backends_params:
+            self.add_ml_backend(ml_backend_params, raise_on_error=False)
 
     def load_converter(self):
-        self.converter = Converter(self.label_config_full)
+        self.converter = Converter(self.parsed_label_config)
 
     @property
     def id(self):
@@ -121,18 +149,44 @@ class Project(object):
     def label_config(self):
         return self.project_obj.label_config
 
+    @property
+    def ml_backends_connected(self):
+        return len(self.ml_backends) > 0
+
+    @property
+    def task_data_login(self):
+        return self.project_obj.task_data_login
+
+    @property
+    def task_data_password(self):
+        return self.project_obj.task_data_password
+
     def extract_data_types(self, config):
         return self.project_obj.extract_data_types(config)
 
     def validate_label_config(self, config_string):
+        logger.debug('Validate label config')
         self.project_obj.validate_label_config(config_string)
 
+        logger.debug('Get parsed config')
         parsed_config = parse_config(config_string)
 
+        logger.debug('Validate label config on derived input schema')
         self.validate_label_config_on_derived_input_schema(parsed_config)
+
+        logger.debug('Validate label config on derived output schema')
         self.validate_label_config_on_derived_output_schema(parsed_config)
-        if self.ml_backend:
-            self.ml_backend.validate(config_string)
+
+    def _save_config(self):
+        with io.open(self.config['config_path'], mode='w') as f:
+            json.dump(self.config, f, indent=2)
+
+    def update_params(self, params):
+        if 'ml_backend' in params:
+            ml_backend_params = self._create_ml_backend_params(params['ml_backend'])
+            self.add_ml_backend(ml_backend_params)
+            self.config['ml_backends'].append(ml_backend_params)
+            self._save_config()
 
     def update_label_config(self, new_label_config):
         label_config_file = self.config['label_config']
@@ -162,7 +216,7 @@ class Project(object):
         """
         for result in completion['result']:
             result_type = result.get('type')
-            if result_type == 'relation':
+            if result_type in ('relation', 'rating', 'pairwise'):
                 continue
             if 'from_name' not in result or 'to_name' not in result:
                 logger.error('Unexpected completion.result format: "from_name" or "to_name" not found in %r' % result)
@@ -261,8 +315,9 @@ class Project(object):
                 json.dump({}, f)
 
         # delete everything on ML backend
-        if self.ml_backend:
-            self.ml_backend.clear(self)
+        if self.ml_backends_connected:
+            for m in self.ml_backends:
+                m.clear(self)
 
         # reload everything related to tasks
         self.load_tasks()
@@ -302,15 +357,22 @@ class Project(object):
             return None
         return self.tasks.get(task_id)
 
+    def iter_completions(self):
+        root_dir = self.config['output_dir']
+        os.mkdir(root_dir) if not os.path.exists(root_dir) else ()
+        files = os.listdir(root_dir)
+        for f in files:
+            if f.endswith('.json'):
+                yield os.path.join(root_dir, f)
+
     def get_completions_ids(self):
         """ List completion ids from output_dir directory
 
         :return: filenames without extensions and directories
         """
-        root_dir = self.config['output_dir']
-        os.mkdir(root_dir) if not os.path.exists(root_dir) else ()
-        files = os.listdir(root_dir)
-        completions = [int(os.path.splitext(f)[0]) for f in files if f.endswith('.json')]
+        completions = []
+        for f in self.iter_completions():
+            completions.append(int(os.path.splitext(os.path.basename(f))[0]))
         logger.debug('{num} completions found in {output_dir}'.format(
             num=len(completions), output_dir=self.config["output_dir"]))
         return sorted(completions)
@@ -380,7 +442,11 @@ class Project(object):
             completion['id'] = task['id'] * 1000 + len(task['completions']) + 1
             task['completions'].append(completion)
 
-        self._update_derived_output_schema(completion)
+        try:
+            self._update_derived_output_schema(completion)
+        except Exception as exc:
+            logger.error(exc, exc_info=True)
+            logger.debug(json.dumps(completion, indent=2))
 
         # write task + completions to file
         filename = os.path.join(self.config['output_dir'], str(task_id) + '.json')
@@ -399,6 +465,32 @@ class Project(object):
         self.load_tasks()
         self.load_derived_schemas()
 
+    def make_predictions(self, task):
+        task = deepcopy(task)
+        task['predictions'] = []
+        try:
+            for ml_backend in self.ml_backends:
+                if not ml_backend.connected:
+                    continue
+                predictions = ml_backend.make_predictions(task, self)
+                predictions['created_by'] = ml_backend.model_name
+                task['predictions'].append(predictions)
+        except Exception as exc:
+            logger.debug(exc)
+        return task
+
+    def train(self):
+        completions = []
+        for f in self.iter_completions():
+            completions.append(json_load(f))
+        train_status = False
+        if self.ml_backends_connected:
+            for ml_backend in self.ml_backends:
+                if ml_backend.connected:
+                    ml_backend.train(completions, self)
+                    train_status = True
+        return train_status
+
     @classmethod
     def get_project_dir(cls, project_name, args):
         return os.path.join(args.root_dir, project_name)
@@ -446,6 +538,16 @@ class Project(object):
             return task_loader.from_dir_with_audio_files(input_path, data_key)
         raise RuntimeError('Can\'t load tasks for input format={}'.format(args.input_format))
 
+    @classmethod
+    def _create_ml_backend_params(cls, url):
+        if '=http' in url:
+            name, url = url.split('=', 1)
+        else:
+            name = str(uuid4())[:8]
+        if not is_url(url):
+            raise ValueError('Specified string "' + url + '" doesn\'t look like URL.')
+        return {'url': url, 'name': name}
+
     @classmethod
     def create_project_dir(cls, project_name, args):
         """
@@ -519,16 +621,18 @@ class Project(object):
             print(completions_dir + ' output dir has been created.')
         config['output_dir'] = 'completions'
 
-        if args.ml_backend_url:
-            if 'ml_backend' not in config or not isinstance(config['ml_backend'], dict):
-                config['ml_backend'] = {}
-            config['ml_backend']['url'] = args.ml_backend_url
-            if args.ml_backend_name:
-                config['ml_backend']['name'] = args.ml_backend_name
-            else:
-                config['ml_backend']['name'] = str(uuid4())
+        if 'ml_backends' not in config or not isinstance(config['ml_backends'], list):
+            config['ml_backends'] = []
+        if args.ml_backends:
+            for url in args.ml_backends:
+                config['ml_backends'].append(cls._create_ml_backend_params(url))
 
-        config['sampling'] = args.sampling
+        if args.sampling:
+            config['sampling'] = args.sampling
+        if args.port:
+            config['port'] = args.port
+        if args.host:
+            config['host'] = args.host
 
         # create config.json
         config_json = 'config.json'
@@ -544,7 +648,11 @@ class Project(object):
         return dir
 
     @classmethod
-    def _get_config(cls, project_dir, args):
+    def get_config(cls, project_name, args):
+        return cls._get_config(cls.get_project_dir(project_name, args))
+
+    @classmethod
+    def _get_config(cls, project_dir, args=None):
         """
         Get config from input args Namespace acquired by Argparser
         :param args:
@@ -552,18 +660,20 @@ class Project(object):
         """
         # check if project directory exists
         if not os.path.exists(project_dir):
+            project_name = args.project_name if args is not None else '<project_name>'
             raise FileNotFoundError(
                 'Couldn\'t find directory ' + project_dir +
                 ', maybe you\'ve missed appending "--init" option:\nlabel-studio start ' +
-                args.project_name + ' --init'
+                project_name + ' --init'
             )
 
         # check config.json exists in directory
         config_path = os.path.join(project_dir, 'config.json')
         if not os.path.exists(config_path):
+            project_name = args.project_name if args is not None else '<project_name>'
             raise FileNotFoundError(
                 'Couldn\'t find config file ' + config_path + ' in project directory ' + project_dir +
-                ', maybe you\'ve missed appending "--init" option:\nlabel-studio start ' + args.project_name + ' --init'
+                ', maybe you\'ve missed appending "--init" option:\nlabel-studio start ' + project_name + ' --init'
             )
 
         config_path = os.path.abspath(config_path)
