@@ -64,11 +64,11 @@ def exception_treatment(f):
 
         except Exception as e:
             traceback = tb.format_exc()
-
+            logger.debug(traceback)
             body = {'traceback': traceback}
             if hasattr(exception_f, 'request_id'):
                 body['request_id'] = exception_f.request_id
-            return answer(501, str(e), body)
+            return answer(500, str(e), body)
 
     exception_f.__name__ = f.__name__
     return exception_f
@@ -108,11 +108,25 @@ def get_app_version():
 
 def parse_config(config_string):
 
+    LABEL_TAGS = {'Label', 'Choice'}
+    NOT_CONTROL_TAGS = {'Filter',}
+
     def _is_input_tag(tag):
         return tag.attrib.get('name') and tag.attrib.get('value')
 
     def _is_output_tag(tag):
-        return tag.attrib.get('name') and tag.attrib.get('toName')
+        return tag.attrib.get('name') and tag.attrib.get('toName') and tag.tag not in NOT_CONTROL_TAGS
+
+    def _get_parent_output_tag_name(tag, outputs):
+        # Find parental <Choices> tag for nested tags like <Choices><View><View><Choice>...
+        parent = tag
+        while True:
+            parent = parent.getparent()
+            if parent is None:
+                return
+            name = parent.attrib.get('name')
+            if name in outputs:
+                return name
 
     xml_tree = etree.fromstring(config_string)
 
@@ -122,11 +136,17 @@ def parse_config(config_string):
             outputs[tag.attrib['name']] = {'type': tag.tag, 'to_name': tag.attrib['toName'].split(',')}
         elif _is_input_tag(tag):
             inputs[tag.attrib['name']] = {'type': tag.tag, 'value': tag.attrib['value'].lstrip('$')}
-        parent = tag.getparent()
-        if parent is not None and parent.attrib.get('name') in outputs:
-            actual_value = tag.attrib.get('alias') or tag.attrib['value']
-            labels[parent.attrib['name']].add(actual_value)
-
+        if tag.tag not in LABEL_TAGS:
+            continue
+        parent_name = _get_parent_output_tag_name(tag, outputs)
+        if parent_name is not None:
+            actual_value = tag.attrib.get('alias') or tag.attrib.get('value')
+            if not actual_value:
+                logger.debug(
+                    'Inspecting tag {tag_name}... found no "value" or "alias" attributes.'.format(
+                        tag_name=etree.tostring(tag, encoding='unicode').strip()[:50]))
+            else:
+                labels[parent_name].add(actual_value)
     for output_tag, tag_info in outputs.items():
         tag_info['inputs'] = []
         for input_tag_name in tag_info['to_name']:
@@ -136,6 +156,7 @@ def parse_config(config_string):
                                .format(input_tag_name=input_tag_name, output_tag=output_tag))
             tag_info['inputs'].append(inputs[input_tag_name])
         tag_info['labels'] = list(labels[output_tag])
+    logger.debug('Parsed config:\n' + json.dumps(outputs, indent=2))
     return outputs
 
 
@@ -153,7 +174,7 @@ def get_config_templates():
     """ Get label config templates from directory (as usual 'examples' directory)
     """
     from collections import defaultdict, OrderedDict
-    templates = defaultdict(list)
+    templates = defaultdict(lambda: defaultdict(list))
 
     for i, path in enumerate(iter_config_templates()):
         # open and check xml
@@ -176,12 +197,23 @@ def get_config_templates():
         meta['label_config'] = '-->\n'.join(code.split('-->\n')[1:])  # remove all comments at the beginning of code
 
         meta['category'] = meta['category'] if 'category' in meta else 'no category'
-        templates[meta['category']].append(meta)
+        meta['complexity'] = meta['complexity'] if 'complexity' in meta else 'no complexity'
+        templates[meta['complexity']][meta['category']].append(meta)
 
     # sort by title
+    ordering = {
+        'basic': ['audio', 'image', 'text', 'html', 'other'],
+        'advanced': ['layouts', 'nested', 'per-region', 'other']
+    }
     ordered_templates = OrderedDict()
-    for key in sorted(templates.keys()):
-        ordered_templates[key] = sorted(templates[key], key=lambda x: x['title'])
+    for complexity in ['basic', 'advanced']:
+        ordered_templates[complexity] = OrderedDict()
+        # add the rest from categories not presented in manual ordering
+        x, y = ordering[complexity], templates[complexity].keys()
+        ordering[complexity] = x + list((set(x) | set(y)) - set(x))
+        for category in ordering[complexity]:
+            sort = sorted(templates[complexity][category], key=lambda z: z.get('order', None) or z['title'])
+            ordered_templates[complexity][category] = sort
 
     return ordered_templates
 
